@@ -401,6 +401,7 @@ class SysrepoSession:
         callback: OperDataCallbackType,
         *,
         no_thread: bool = False,
+        oper_merge: bool = False,
         private_data: Any = None,
         asyncio_register: bool = False,
         strict: bool = False,
@@ -420,6 +421,11 @@ class SysrepoSession:
         :arg no_thread:
             There will be no thread created for handling this subscription meaning no
             event will be processed! Default to `True` if asyncio_register is `True`.
+        :arg oper_merge:
+            Instead of removing any previous existing matching data before
+            getting them from an operational subscription callback, keep
+            them. Then the returned data are merged into the existing
+            data. Valid only for operational subscriptions.
         :arg private_data:
             Private context passed to the callback function, opaque to sysrepo.
         :arg asyncio_register:
@@ -448,7 +454,7 @@ class SysrepoSession:
 
         if asyncio_register:
             no_thread = True  # we manage our own event loop
-        flags = _subscribe_flags(no_thread=no_thread)
+        flags = _subscribe_flags(no_thread=no_thread, oper_merge=oper_merge)
 
         check_call(
             lib.sr_oper_get_subscribe,
@@ -978,6 +984,7 @@ class SysrepoSession:
         no_subs: bool = False,
         no_stored: bool = False,
         strip_prefixes: bool = True,
+        include_implicit_defaults: bool = False,
         trim_default_values: bool = False,
         keep_empty_containers: bool = False,
     ) -> Dict:
@@ -1003,6 +1010,7 @@ class SysrepoSession:
                 with_siblings=True,
                 absolute=True,
                 strip_prefixes=strip_prefixes,
+                include_implicit_defaults=include_implicit_defaults,
                 trim_default_values=trim_default_values,
                 keep_empty_containers=keep_empty_containers,
             )
@@ -1042,6 +1050,29 @@ class SysrepoSession:
         """
         check_call(lib.sr_delete_item, self.cdata, str2c(xpath), 0)
 
+    def delete_oper_item(self, xpath: str, value: Any = None) -> None:
+        """
+        Prepare to delete the nodes matching the specified xpath in the operational
+        datastore. These changes are applied only after calling
+        apply_changes().
+
+        :arg xpath:
+            Path identifier of the data element to be deleted.
+        :arg value:
+            String representation of the value deleted.
+
+        :raises SysrepoNotFoundError:
+            If no nodes match the path.
+        """
+        if value is not None and not isinstance(value, str):
+            if isinstance(value, bool):
+                value = str(value).lower()
+            else:
+                value = str(value)
+        check_call(
+            lib.sr_oper_delete_item_str, self.cdata, str2c(xpath), str2c(value), 0
+        )
+
     def edit_batch_ly(
         self, edit: libyang.DNode, default_operation: str = "merge"
     ) -> None:
@@ -1380,7 +1411,9 @@ def _get_oper_flags(no_state=False, no_config=False, no_subs=False, no_stored=Fa
 
 
 # -------------------------------------------------------------------------------------
-def _subscribe_flags(no_thread=False, passive=False, done_only=False, enabled=False):
+def _subscribe_flags(
+    no_thread=False, passive=False, done_only=False, enabled=False, oper_merge=False
+):
     flags = 0
     if no_thread:
         flags |= lib.SR_SUBSCR_NO_THREAD
@@ -1390,6 +1423,8 @@ def _subscribe_flags(no_thread=False, passive=False, done_only=False, enabled=Fa
         flags |= lib.SR_SUBSCR_DONE_ONLY
     if enabled:
         flags |= lib.SR_SUBSCR_ENABLED
+    if oper_merge:
+        flags |= lib.SR_SUBSCR_OPER_MERGE
     return flags
 
 
